-module(ksudoku).
-author(hanak@inf.bme.hu).
-vsn('2008-11-12').

% -compile(export_all).
-export([sudoku_be/1,sudoku_ki/2,megold/2,stopper/2]).

-import(sudoku,[sudoku/1]).

%% @type sspec() = {size(), board()}.
%% @type size()  = integer().
%% @type field() = [info()].
%% @type info()  = e | o | s | w | integer().
%% @type board() = [[field()]].

%% @type ssol() = [[integer()]].

%% @spec sudoku:sudoku(SSpec::sspec()) -> SSols::[ssol()].
%% @doc  SSols az SSpec feladvnyt kielgt megoldsok listja.

%% ksudoku-specifikcik

%% @spec sudoku_be(FileIn::string()) -> SSpec::sspec().
%% @doc  SSpec a FileIn szvegfjlbl beolvasott Sudoku-feladvny.
%%
sudoku_be(FileIn) ->
    {ok, InputFile} = file:open(FileIn,read),
    {N,_} = case io:get_line(InputFile,'') of
		eof ->
		    io:format("Ures a(z) ~s fajl.~n",[FileIn]),
		    exit('Hibas feladvany');
		I ->
		    string:to_integer(cleaned(I))
	     end,
    KT = tabla_be(InputFile,N),
    file:close(InputFile),
    case helyes_feladvany(KT) of
	true ->
	    KT;
	false ->
	    io:format("Hibas a feladvany a(z) ~s fajlban.~n",[FileIn]),
	    exit('Hibas feladvany')
    end.

tabla_be(InputFile,M) ->
    {M,tabla_be(InputFile,M*M,[])}.

tabla_be(_InputFile,0,Zss) ->
    lists:reverse(Zss);
tabla_be(InputFile,Meret,Zss) ->
    case io:get_line(InputFile,'') of
	eof ->
	    io:format("Tul keves sor van a input fajlban.~n",[]),
		    exit('Hibas feladvany');
	Sor ->
	    Ss = string:tokens(cleaned(Sor)," "),
	    Zs = lists:map(fun str2infolist/1, Ss),
	    case Zs of
		[] ->
		    tabla_be(InputFile,Meret,Zss);
		_ ->
		    tabla_be(InputFile,Meret-1,[Zs|Zss])
	    end
    end.

cleaned(Sor) ->
    S0 = lists:filter(fun(C) -> C /= $\t end,Sor),
    string:strip(string:strip(S0),right,$\n).

str2infolist([$-]) ->
    [];
str2infolist(Xs) ->
    str2infolist2(Xs).

str2infolist2(XXs=[X|_Xs]) when $0=<X andalso X=<$9 ->
    {I,Rs} = string:to_integer(XXs),
    [I|str2infolist2(Rs)];
str2infolist2([X|Xs]) ->
    [list_to_atom([X])|str2infolist2(Xs)];
str2infolist2([]) ->
    [].

%% @spec helyes_feladvany(SSpec::sspec()) -> B::bool().
%% @doc  B igaz, ha a feladvny megfelel a specifikcinak.
%%
helyes_feladvany({K,T}) ->
    KK = K*K,
    % a cellamret j
    (1 =< K) andalso (K =< 5) andalso
    % a sorok s soronknt az oszlopok szma j 
    (KK == length(T)) andalso
	(lists:all(fun(L) -> length(L) == KK end,T)) andalso
    % a tbla megfelel a sudoku-feladvny specifikcijnak
%    lists:all(fun helyes_mezo/2,lists:concat(T)).
    lists:all(fun(Fs) -> helyes_mezo(KK,Fs) end,lists:concat(T)).

helyes_mezo(KK,Fs) ->
    (length(lists:filter(fun erlang:is_integer/1,Fs)) < 2) andalso
	helyes_info(KK,Fs).

helyes_info(_KK,[]) ->
    true;
helyes_info(KK,[e|Fs]) ->
    helyes_info(KK,Fs);
helyes_info(KK,[o|Fs]) ->
    helyes_info(KK,Fs);
helyes_info(KK,[s|Fs]) ->
    helyes_info(KK,Fs);
helyes_info(KK,[w|Fs]) ->
    helyes_info(KK,Fs);
helyes_info(KK,[F|Fs])
  when is_integer(F) andalso (1=<F) andalso (F=<KK) -> 
    helyes_info(KK,Fs);
helyes_info(_KK,[_|_]) ->
    false.

%% @spec ksudoku:sudoku_ki(FileOut::string(),SSols::[ssol()]) -> void()
%% @doc  Az SSols listban tadott Sudoku-megoldsokat kirja
%%       a FileOut szvegfjlba.
%%
sudoku_ki(FileOut,SSols) ->
    {ok,OutputFile} = file:open(FileOut,write),
    tablak_ki(OutputFile,SSols),
    file:close(OutputFile).

tablak_ki(OutputFile,[SSol]) ->
    io:fwrite(OutputFile,"~s~n",[tabla_ki(SSol)]);
tablak_ki(OutputFile,[SSol|SSols]) ->
    io:fwrite(OutputFile,"~s~n~n",[tabla_ki(SSol)]),
    tablak_ki(OutputFile,SSols);
tablak_ki(OutputFile,[]) ->
    io:fwrite(OutputFile,"",[]).

tabla_ki([L]) ->
    sor_ki(L);
tabla_ki([L|Ls]) ->
    sor_ki(L) ++ "\n" ++ tabla_ki(Ls);
tabla_ki(_) ->
    [].

sor_ki([H|T]) ->
    I2L = fun integer_to_list/1,
    I2L(H) ++ lists:foldr(fun(A,B) -> " " ++ I2L(A) ++ B end,"",T);
sor_ki([]) ->
    "".

%% @spec ksudoku:megold(FileIn::string(),FileOut::string()) -> void()
%% @doc  Beolvas egy feladvnyt a FileIn szvegfjlbl s sszes
%%       megoldst kirja a FileOut szvegfjlba. Ehhez
%%       felhasznlja a sudoku:sudoku/1 fggvnyt.
%%
megold(FileIn,FileOut) ->
    KT = sudoku_be(FileIn),
    SSols = sudoku:sudoku(KT), 
    sudoku_ki(FileOut,SSols).

%% @spec ksudoku:stopper(FileIn::string(),FileOut::string()) -> void()
%% @doc  Mint ksudoku:megold/2, de a vgn kirja a FileIn nevet,
%%       a megoldsok szmt s a futsi idt is.
%%
stopper(FileIn,FileOut) ->
    KT = sudoku_be(FileIn),
    T1 = now(),
    SSols = sudoku:sudoku(KT),
    T2 = now(),
    T = timer:now_diff(T2,T1),
    sudoku_ki(FileOut,SSols),
    io:format("Feladvany: ~s, megoldas: ~w, futasi ido: ~w ms\n",
	      [FileIn,length(SSols),T div 1000]).
